function options_ip = getIpOptions(options,nVar,mEq,nonlconflag,defaultopt,defaultHessMemory,defaultPivotThreshold)
%getIpOptions Helper function that creates structure of internal options 
% options_ip based on the values in user-supplied structure "options" and 
% on defaultopt

%   Copyright 2007 The MathWorks, Inc.
%   $Revision: 1.1.6.2 $  $Date: 2007/12/10 21:50:29 $

% Read in all options fields that don't need processing
fieldsThatDontNeedProcessing = {'AlwaysHonorConstraints';'DerivativeCheck'; ...
    'Diagnostics';'DiffMaxChange';'DiffMinChange';'Display';'FinDiffType'; ...
    'FunValCheck';'GradConstr';'GradObj';'HessFcn';'HessMult';'InitBarrierParam'; ...
    'MaxIter';'ObjectiveLimit';'OutputFcn';'PlotFcns';'ScaleProblem'; ...
    'TolCon';'TolFun';'TolGradCon';'TolProjCG';'TolProjCGAbs';'TolX'};

for i = 1:numel(fieldsThatDontNeedProcessing)
    field = fieldsThatDontNeedProcessing{i};
    options_ip.(field) = optimget(options,field,defaultopt,'fast');
end

% Now read-in all fields that can be either a string or a numeric value
% Each row of cell array below contains: 
% field name, allowed string, equivalent numeric value, datatype (with appropriate article)
% for error message
numericOrStringFields = ... % NB: This is a *2-dimensional* cell-array
    {'MaxFunEvals','100*numberofvariables',100*nVar,'an integer';
    'InitTrustRegionRadius','sqrt(numberofvariables)',sqrt(nVar),'a real';
    'MaxProjCGIter','2*(numberofvariables-numberofequalities)',max(2*(nVar-mEq),0),'a matrix';
    'TypicalX','ones(numberofvariables,1)',ones(nVar,1),'a real matrix'};

for i = 1:size(numericOrStringFields,1)
    field = numericOrStringFields{i,1};
    allowedString = numericOrStringFields{i,2};
    equivNumValue = numericOrStringFields{i,3};
    dataType = numericOrStringFields{i,4};
    options_ip.(field) = getNumericOrStringField(field,allowedString, ...
        equivNumValue,dataType,options,defaultopt);
end

% Now read-in fields that need special processing

% Option Hessian can be either a string or a cell array. So as not
% to have to check for this inside barrier.m at every iteration,
% we create one or two internal options: HessType and HessMemory.
Hessian = optimget(options,'Hessian',defaultopt,'fast');
options_ip.HessMemory = defaultHessMemory;
if ischar(Hessian)
    % If Hessian user-supplied, on, fin-diff-grads, gradients must be on
    if ( strcmpi(Hessian,'user-supplied') || strcmpi(Hessian,'on') ...
            || strcmpi(Hessian,'fin-diff-grads') ) && ...
            ( strcmpi(options_ip.GradObj,'off') || nonlconflag && strcmpi(options_ip.GradConstr,'off') )
        mexcptn = MException('optim:getIpOptions:HessButNoGrads', ...
            ['Must set GradObj = ''on'' and (if nonlinear constraints are present)\n' ...
            'GradConstr = ''on'' in order to use option Hessian = ''%s''.'],Hessian);
        throwAsCaller(mexcptn);
    end    
    if strcmpi(Hessian,'off')
        % No Hessian function provided and old value; set to default
        options_ip.HessType = defaultopt.Hessian;
    elseif strcmpi(Hessian,'user-supplied') || strcmpi(Hessian,'on')
        % Hessian provided, either via options HessFcn or HessMult
        if ~isempty(options.HessFcn) 
            options_ip.HessType = 'user-supplied';
            if ~isempty(options.HessMult)
                % Both HessFcn and HessMult provided; honor HessFcn
                warning('optim:getIpOptions:BothHessFcnHessMult', ...
                    'Both HessFcn and HessMult provided; using HessFcn and ignoring HessMult.')
            end
        elseif ~isempty(options.HessMult)
            options_ip.HessType = 'hessmult';
        else 
            % Hessian user-supplied but no HessFcn nor HessMult
            mexcptn = MException('optim:getIpOptions:BadHessOptions', ...
                ['Hessian option set to ''%s'' but no Hessian function provided' ...
                ' in options HessFcn nor in HessMult.'],Hessian);
            throwAsCaller(mexcptn);
        end
    else % any of the current legal string values for Hessian option
        options_ip.HessType = Hessian;
    end
elseif iscell(Hessian)
    options_ip.HessType = Hessian{1};
    if numel(Hessian) > 1
        options_ip.HessMemory = Hessian{2};
    end
end

% SubproblemAlgorithm corresponds to three internal options: IpAlgorithm,
% LinearSystemSolver, and PivotThreshold.
% The value of SubproblemAlgorithm can be either a string or a cell array
SubproblemAlgorithm = optimget(options,'SubproblemAlgorithm',defaultopt,'fast');

% Store subproblem algorithm in a string for easy processing
% and read-in pivot threshold
% Because of error checking in options functions, SubproblemAlgorithm
% can only be either a string or a cell array.
if ischar(SubproblemAlgorithm)
    SubproblemAlgorithm_string = SubproblemAlgorithm;
    options_ip.PivotThreshold = defaultPivotThreshold; 
elseif iscell(SubproblemAlgorithm)
    SubproblemAlgorithm_string = SubproblemAlgorithm{1};
    if numel(SubproblemAlgorithm) > 1
        % Pivot threshold passed in with option
        options_ip.PivotThreshold = SubproblemAlgorithm{2};
    else
        options_ip.PivotThreshold = defaultPivotThreshold; 
    end
end

if strcmpi(SubproblemAlgorithm_string,'ldl-factorization')
    options_ip.IpAlgorithm = 'direct';
    options_ip.LinearSystemSolver = 'ldl-factorization';
elseif strcmpi(SubproblemAlgorithm_string,'lu-eig-factorization') 
    options_ip.IpAlgorithm = 'direct';
    options_ip.LinearSystemSolver = 'lu-eig-factorization';
elseif strcmpi(SubproblemAlgorithm_string,'cg') 
    options_ip.IpAlgorithm = 'cg';
    options_ip.LinearSystemSolver = 'ldl-factorization';
elseif strcmpi(SubproblemAlgorithm_string,'cg-lu')
    options_ip.IpAlgorithm = 'cg';
    options_ip.LinearSystemSolver = 'lu-eig-factorization';
end

% Must select SubproblemAlgorithm = 'cg' in order to user fin-diff-grads or HessMult
if ~strcmpi(options_ip.IpAlgorithm,'cg')
    if strcmpi(options_ip.HessType,'fin-diff-grads')
        mexcptn = MException('optim:getIpOptions:FinDiffGradsAndNotCG', ...
            ['In order to use option Hessian = ''fin-diff-grads'', option SubproblemAlgorithm\n' ...
            ' must be set to ''cg''.']);
        throwAsCaller(mexcptn);
    elseif strcmpi(options_ip.HessType,'hessmult')
        mexcptn = MException('optim:getIpOptions:HessMultAndNotCG', ...
            'In order to use option HessMult, option SubproblemAlgorithm must be set to ''cg''.');
        throwAsCaller(mexcptn);
    end
end
%--------------------------------------------------------------------
function fieldValue = getNumericOrStringField(fieldString,allowedString, ...
    equivNumValue,dataTypeWithArticleForErrMsg,options,defaultopt)
%getNumericOrStringField reads in field fieldString from options/defaultopt.
% If value that is read in is a string, it compares it to allowedString
% and converts it to equivalent numeric value. If string is not
% allowedString, errors with appropriate error ID and error message.
% Note: the input dataTypeWithArticleForErrMsg is the data type of the
% the field along with its article for error printing purposes. E.g.
% 'an integer', 'a real', or 'a matrix'.

% Read in value of field
fieldValue = optimget(options,fieldString,defaultopt,'fast');
if ischar(fieldValue)
    if isequal(lower(fieldValue),lower(allowedString))
        fieldValue = equivNumValue;
    else
        errId = ['optim:getIpOptions:Invalid' fieldString 'Option'];
        error(errId,'Option %s must be %s value if not the default.', ...
            fieldString,dataTypeWithArticleForErrMsg)              
    end
end
